class UTFChar;
class UTFString;

#ifndef UTFTEXTBUFFER
#define UTFTEXTBUFFER

#pragma export on

/*
	CLASS UTFTextBuffer
	
	A class that allows the use and modification of a text buffer.
	Provides a modest wordprocessor type functionality that just needs
	a little UI.
*/
class UTFTextBuffer : public BLocker
{
public:
	//Constructors
	UTFTextBuffer();
	UTFTextBuffer(BDataIO* inbuf);
	UTFTextBuffer(const UTFTextBuffer& inbuf);
	
	//Destructor
	virtual ~UTFTextBuffer();

	//Copy Operator
	UTFTextBuffer& operator=(const UTFTextBuffer& ibuf);
	
	//Methods
	
	/*
		LoadData
		
		Load data into the text buffer from a BDataIO* derived
		class
	*/
	void LoadData(BDataIO* loadbuf);

	/*
		SaveData
		
		Save data into a BDataIO* derived class
	*/
	void SaveData(BDataIO* savebuf);
	
	/*
		Set/GetCaret
		 
		Set/Return the current Caret position. The caret position
		determins where new characters are inserted at or removed
		from.
	*/	
	void SetCaret(uint32 Row, uint32 Col);
	void GetCaret(uint32& Row, uint32& Col);
	
	/*
		Caret*
		
		Adjust caret by one in either Up, Down, Left or right.
		
		0 <= CaretPosRow < buffer row count
		
		if CaretPsCol < 0, it is wrapped to the end of the next row up.
		if CaretPosX > Col width of that row, then the Caret is set to col0
		of the next Row down. 
	*/
	void CaretUp();
	void CaretDown();
	void CaretLeft();
	void CaretRight();
	
	/*
		SetSelectionBegin/End
		
		Set the beginning and ending of the current selection.
		NOTE: No checking is done to make sure the Begining
		selection point is ahead of the ending point. However, methods
		such as Cut/Copy/Paste will only do something if the selection range
		is correct.
	*/
	void SetSelectionBegin(uint32 Row, uint32 Col);
	void SetSelectionEnd(uint32 Row, uint32 Col);
	
	/*
		ResetSelection
		
		Sets the beginning and ending selection range to the
		current caret position.
	*/
	void ResetSelection();
	
	/*
		InsertChar
		
		Inserts a character into the current caret position
		(or replaces the current selection with the character).
	*/
	void InsertChar(const UTFChar& InsChar);
	
	/*
		RemoveChar
		
		Removes the character at the current caret position.
		If a selection exists, it is cut as well.
	*/
	UTFChar RemoveChar();
	
	/*
		Cut/Copy/Paste
		
		Standard processing comamnds. Works on the current selection.
	*/
	UTFString Cut();
	UTFString Copy();
	void Paste(const UTFString& PasteString);
	
	/*
		AddRow
		
		Adds a row either to the end of the buffer,
		or into a specific row in the buffer. If the new
		index > than the number of current rows, than the new
		row is added to the end of the buffer.
	*/
	void AddRow(const UTFString& RowString);
	void AddRow(uint32 Row, const UTFString& RowString);
	
	/*
		RemoveRow
		
		Remove a row from a given row position or from the begining
		of the text buffer.
	*/
	UTFString RemoveRow();
	UTFString RemoveRow(uint32 Row);
	
	/*
		NumRows
		
		Return the number of rows in the text buffer.
	*/
	uint32 NumRows();
	
	/*
		Get/SetRow
		
		Get/Set a specific Row To/From a UTFString.
	*/
	UTFString GetRow(uint32 Row);
	void SetRow(uint32 Row, const UTFString& RowString);
		
	/*
		ClearBuffer
		
		Clears the text buffer
	*/
	void ClearBuffer();

protected:
private:

	void CopyTextBuffer(const UTFTextBuffer& tb);
	void ProcessDataBuffer(BDataIO* buf);
	void Init();
	
	//TextBuffer
	BList	m_buffer;
	
	//Synchronizer
	BLocker	m_lock;

	//Caret
	uint32 	m_carposx;
	uint32	m_carposy;
	
	//Selection Variables	
	uint32	m_selstartx;
	uint32	m_selstarty;
	uint32	m_selendx;
	uint32	m_selendy;
};

#pragma export off

#endif
